'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
var flattenVNodes = function flattenVNodes(children) {
  var result = [];

  var traverse = function traverse(children) {
    if (Array.isArray(children)) {
      children.forEach(function (child) {
        if (vue.isVNode(child)) {
          var _child$component;

          result.push(child);

          if ((_child$component = child.component) !== null && _child$component !== void 0 && _child$component.subTree) {
            traverse(child.component.subTree.children);
          }

          if (child.children) {
            traverse(child.children);
          }
        }
      });
    }
  };

  traverse(children);
  return result;
};
var sortChildren = function sortChildren(parent, publicChildren, internalChildren) {
  var VNodes = flattenVNodes(parent.subTree.children);
  internalChildren.sort( // eslint-disable-next-line comma-dangle
  function (a, b) {
    return VNodes.indexOf(a.vnode) - VNodes.indexOf(b.vnode);
  });
  var orderedPublicChildren = internalChildren.map(function (item) {
    return item.proxy;
  });
  publicChildren.sort(function (a, b) {
    var indexA = orderedPublicChildren.indexOf(a);
    var indexB = orderedPublicChildren.indexOf(b);
    return indexA - indexB;
  });
};
var createProvider = function createProvider(key) {
  var publicChildren = vue.reactive([]);
  var internalChildren = vue.reactive([]);
  var parent = vue.getCurrentInstance();

  var provider = function provider(value) {
    var link = function link(child) {
      if (child.proxy) {
        internalChildren.push(child);
        publicChildren.push(child.proxy);
        sortChildren(parent, publicChildren, internalChildren);
      }
    };

    var unlink = function unlink(child) {
      var idx = internalChildren.indexOf(child);
      publicChildren.splice(idx, 1);
      internalChildren.splice(idx, 1);
    };

    vue.provide(key, _objectSpread({
      link: link,
      unlink: unlink,
      children: publicChildren,
      internalChildren: internalChildren
    }, value));
  };

  return {
    children: publicChildren,
    provider: provider
  };
};

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};
  var target = _objectWithoutPropertiesLoose(source, excluded);
  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

var _excluded = ["link", "unlink", "internalChildren"];

var useProvider = function useProvider(key) {
  var context = vue.inject(key, null);

  if (context) {
    var instance = vue.getCurrentInstance();

    var _link = context.link,
        _unlink = context.unlink,
        internalChildren = context.internalChildren,
        rest = _objectWithoutProperties(context, _excluded);

    _link(instance);

    vue.onUnmounted(function () {
      return _unlink(instance);
    });
    var idx = vue.computed(function () {
      return internalChildren.indexOf(instance);
    });
    return {
      context: rest,
      idx: idx.value
    };
  }

  return {
    context: null,
    idx: -1
  };
};

exports.createProvider = createProvider;
exports.flattenVNodes = flattenVNodes;
exports.sortChildren = sortChildren;
exports.useProvider = useProvider;
