"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createProvider = exports.sortChildren = exports.flattenVNodes = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/esm/defineProperty"));

var _vue = require("vue");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var flattenVNodes = function flattenVNodes(children) {
  var result = [];

  var traverse = function traverse(children) {
    if (Array.isArray(children)) {
      children.forEach(function (child) {
        if ((0, _vue.isVNode)(child)) {
          var _child$component;

          result.push(child);

          if ((_child$component = child.component) !== null && _child$component !== void 0 && _child$component.subTree) {
            traverse(child.component.subTree.children);
          }

          if (child.children) {
            traverse(child.children);
          }
        }
      });
    }
  };

  traverse(children);
  return result;
};

exports.flattenVNodes = flattenVNodes;

var sortChildren = function sortChildren(parent, publicChildren, internalChildren) {
  var VNodes = flattenVNodes(parent.subTree.children);
  internalChildren.sort( // eslint-disable-next-line comma-dangle
  function (a, b) {
    return VNodes.indexOf(a.vnode) - VNodes.indexOf(b.vnode);
  });
  var orderedPublicChildren = internalChildren.map(function (item) {
    return item.proxy;
  });
  publicChildren.sort(function (a, b) {
    var indexA = orderedPublicChildren.indexOf(a);
    var indexB = orderedPublicChildren.indexOf(b);
    return indexA - indexB;
  });
};

exports.sortChildren = sortChildren;

var createProvider = function createProvider(key) {
  var publicChildren = (0, _vue.reactive)([]);
  var internalChildren = (0, _vue.reactive)([]);
  var parent = (0, _vue.getCurrentInstance)();

  var provider = function provider(value) {
    var link = function link(child) {
      if (child.proxy) {
        internalChildren.push(child);
        publicChildren.push(child.proxy);
        sortChildren(parent, publicChildren, internalChildren);
      }
    };

    var unlink = function unlink(child) {
      var idx = internalChildren.indexOf(child);
      publicChildren.splice(idx, 1);
      internalChildren.splice(idx, 1);
    };

    (0, _vue.provide)(key, _objectSpread({
      link: link,
      unlink: unlink,
      children: publicChildren,
      internalChildren: internalChildren
    }, value));
  };

  return {
    children: publicChildren,
    provider: provider
  };
};

exports.createProvider = createProvider;